#[repr(C)]pub struct GlyphExtents {
pub x_bearing: i32,
pub y_bearing: i32,
pub width: i32,
pub height: i32,
}Expand description
Glyph ink extents in font units.
This matches HarfBuzz’s glyph extents layout and semantics.
Fields
x_bearing: i32Horizontal bearing from glyph origin to the left side of the ink box.
y_bearing: i32Vertical bearing from glyph origin to the top of the ink box.
width: i32Width of the glyph ink box.
height: i32Height of the glyph ink box.